
// --------------------------------------------------------- 
//
// GLStuff.cpp - OpenGL/SDL helper functions
//
// --------------------------------------------------------- 

#include <sdl.h>
#include <SDL_syswm.h>
#include "ld48.h"

OSRT  g_OSRT[NUM_OSRTS];
OSRT *g_pCurrentOSRT=NULL;
int   g_FontListBase[2];

// --------------------------------------------------------- 
// Init OpenGL viewport 
// --------------------------------------------------------- 

void GL_SetViewport(int x,int y,int w,int h,bool bOrtho)
{
	glViewport(x,y,w,h);
	glClearColor(0.3f,0.3f,0.3f,0);
	glClearDepth(1);
    glDepthFunc(GL_LESS);				
    glEnable(GL_TEXTURE_2D);

	if (!bOrtho)
	{
		glEnable(GL_DEPTH_TEST);		
		glShadeModel(GL_SMOOTH);		
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();								
		gluPerspective(70.0f,640.0f/480.0f,0.1f,100.0f);
	}
	else
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();			
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();			
		glOrtho(0,WIDTH,HEIGHT,0,-100,100);		
		glDisable(GL_DEPTH_TEST);		
	}

	glMatrixMode(GL_MODELVIEW);
}



// --------------------------------------------------------- 
// SDL/OpenGL Video mode setup 
// --------------------------------------------------------- 

void GL_SetVideoMode(int w,int h,int flags)
{
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_BUFFER_SIZE, BPP);


	if ( SDL_SetVideoMode(w, h, BPP, SDL_OPENGL | flags) == NULL ) 
	{
		fprintf(stderr, "Failed to Init OpenGL: %s\n", SDL_GetError());
		SDL_Quit();
		exit(0);
	}
	else
	{
		SDL_ShowCursor(SDL_DISABLE);
		GL_SetViewport(0,0,w, h,false);
	}

	GL_CreateFonts();
}



// --------------------------------------------------------- 
// GL_SetRenderTarget
// Set the active render target
// --------------------------------------------------------- 

OSRT *GL_SetRenderTarget(OSRT *pTarget)
{
	OSRT *pPrev=g_pCurrentOSRT;
	g_pCurrentOSRT=pTarget;

	if (pPrev->pSDLRenderTarget)
		SDL_GL_UnlockRenderTarget(pPrev->pSDLRenderTarget);

	if (pTarget->pSDLRenderTarget)
		SDL_GL_LockRenderTarget(pTarget->pSDLRenderTarget);

	glViewport(0,0,pTarget->nWidth,pTarget->nHeight);

	return pPrev;
}


void GL_BindRenderTarget(int nTarget)
{
	GL_SetRenderTarget(&g_OSRT[nTarget]);
}


void GL_BindRenderTargetAsTexture(int nTarget)
{
	glBindTexture(GL_TEXTURE_2D,g_OSRT[nTarget].nTexID);
}


// --------------------------------------------------------- 
// GL_InitRenderTarget();
// Create an off-screen render target
// --------------------------------------------------------- 

void GL_InitRenderTarget(OSRT *pDestTarget,OSRT *pTarget,int w, int h)
{
	OSRT *pPrev;	
	pPrev=GL_SetRenderTarget(pDestTarget);	

    // Create the texture that will be bound to the render target 
    glGenTextures(1, &pTarget->nTexID);
    glBindTexture(GL_TEXTURE_2D, pTarget->nTexID);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    		
	pTarget->nHeight=h;
	pTarget->nHeightP2=1;
	while (pTarget->nHeightP2<pTarget->nHeight) 
		   pTarget->nHeightP2*=2;

	pTarget->nWidth=w;
	pTarget->nWidthP2=1;
	while (pTarget->nWidthP2<pTarget->nWidth) 
		   pTarget->nWidthP2*=2;

    // Create the render target 
	pTarget->pSDLRenderTarget = SDL_GL_CreateRenderTarget(pTarget->nWidthP2, pTarget->nHeightP2, NULL);
    if (pTarget->pSDLRenderTarget) 	
	{
		SDL_GL_BindRenderTarget(pTarget->pSDLRenderTarget, pTarget->nTexID);
				
		GL_SetViewport(0,0,WIDTH,HEIGHT,true);		
	}
	
	// Calculate top left tex coords for the used area
	pTarget->u1= ((float)(pTarget->nWidth)) /((float)(pTarget->nWidthP2));
	pTarget->v1= ((float)(pTarget->nHeight))/((float)(pTarget->nHeightP2));

	// Restore context	
	GL_SetRenderTarget(pPrev);	
}





// --------------------------------------------------------- 
// GL_CreateRenderTargets()
// Create render targets
// --------------------------------------------------------- 

void GL_CreateRenderTargets(void)
{
	g_OSRT[OSRT_SCREEN].pSDLRenderTarget=NULL;
	g_OSRT[OSRT_SCREEN].nWidth  = WIDTH;
	g_OSRT[OSRT_SCREEN].nHeight = HEIGHT;
	g_pCurrentOSRT=&g_OSRT[OSRT_SCREEN];

	GL_InitRenderTarget(&g_OSRT[OSRT_SCREEN],&g_OSRT[OSRT_LIGHTING],WIDTH,HEIGHT);	
}




// --------------------------------------------------------- 
// GL_CreateFonts
// Create fonts
// --------------------------------------------------------- 

void GL_CreateFonts(void)
{
	
	// Create fonts (code is from a NeHe tutorial)
	for(int n=0;n<2;n++)
	{
		HFONT	font;					
		HFONT	oldfont;				

		if (n==0)
		{
			font = CreateFont(	22,0,0,0,	FW_REGULAR,FALSE,	FALSE,	FALSE, ANSI_CHARSET,			
								OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,	ANTIALIASED_QUALITY, FF_DONTCARE|DEFAULT_PITCH,		
								"Tahoma");					
		}
		else
		{
			font = CreateFont(	15,0,0,0,	FW_REGULAR,FALSE,	FALSE,	FALSE, ANSI_CHARSET,			
								OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,	ANTIALIASED_QUALITY, FF_DONTCARE|DEFAULT_PITCH,		
								"Tahoma");					
		}

		SDL_SysWMinfo wmi;
        SDL_VERSION(&wmi.version);
        SDL_GetWMInfo(&wmi);         

		HDC hDC=GetDC(wmi.window);
		oldfont = (HFONT)SelectObject(hDC, font);    

		g_FontListBase[n]=glGenLists(96);			
		wglUseFontBitmaps (hDC, 32,96,g_FontListBase[n]); 

		SelectObject(hDC, oldfont);					
		DeleteObject(font);							
	}
}


// --------------------------------------------------------- 
// GL_CreateFonts
// Draw a string
// --------------------------------------------------------- 

void GL_PrintText(char *string, float x,float y,int nFont)
{
	glDisable(GL_TEXTURE_2D);
				
	glRasterPos2f(x,y); 
	glListBase(g_FontListBase[nFont] - 32);		
	glCallLists (strlen(string), GL_UNSIGNED_BYTE, string); 
	
	glEnable(GL_TEXTURE_2D);
}