
#include "LD48.h"


char *szLevelNames[]={"","At The Beginning","The Dark Place","Hidden Dangers","Minor Mayhem","That's All, Folks!",""};

void CGame::NewGame()
{

	nLevel=0;
	nLives=3;
	nKills=nEnemies=nScore=0;
	nState=STATE_TITLE;
}




void CGame::Init(void)
{
	SFX_Load(SFX_JUMP,"data\\sound\\jump.wav");
	SFX_Load(SFX_LAND,"data\\sound\\land.wav");
	SFX_Load(SFX_IMPACT,"data\\sound\\impact.wav");
	SFX_Load(SFX_SHOOT,"data\\sound\\shoot.wav");
	SFX_Load(SFX_EXPLODE,"data\\sound\\explode.wav");
	
	InitEntities();
	NewGame();
	InitLevel();
}


void CGame::InitLevel(void)
{
	nKills=nEnemies=0;

	ResetEntities();

	FILE *f;
	char szName[64];
	sprintf(szName,"data\\%d.map",nLevel);
	f=fopen(szName,"rb");
	for(int y=0;y<MAP_WIDTH;y++)
	{
		for(int x=0;x<MAP_WIDTH;x++)
		{
			short n;
			fread(&n,2,1,f);
			if(n==0) n=1;
			gMap[x][y]=n-1;
		}			
	}
	fclose(f);
	
	for(int y=0;y<MAP_WIDTH;y++)
	{
		for(int x=0;x<MAP_WIDTH;x++)
		{									

			float fx=(x*32)+16;
			float fy=(y*32)+19;
			if (gMap[x][y]==8)
			{
				ClearTile(fx,fy);
				pPlr=new Player(sprPlayer,&sprTurret,fx,fy);
				fScrollX= pPlr->vPos.x-320;
				fScrollY= pPlr->vPos.y-240;
				AddEntity(pPlr);
			}

			
			if (gMap[x][y]==12)
			{
				ClearTile(fx,fy);
				LightSource *pLight=new LightSource(&sprCeilLight,0,fx,fy-12);				
				
				AddEntity(pLight);
			}

			if (gMap[x][y]==9)
			{
				ClearTile(fx,fy);
				LightSource *pLight=new LightSource(&sprSpinLight,1,fx,fy);				
				AddEntity(pLight);
			}

				
			if (gMap[x][y]==13)
			{
				ClearTile(fx,fy);
				Enemy *pNME=new Enemy(sprFloater,ENEMY_FLOATER,fx,fy);				
				AddEntity(pNME);
				nEnemies++;
			}

			if (gMap[x][y]==14)
			{
				ClearTile(fx,fy);
				Enemy *pNME=new Enemy(sprCrawler,ENEMY_CRAWLER,fx,fy);				
				AddEntity(pNME);
				nEnemies++;
			}

			
			if (gMap[x][y]==15)
			{
				ClearTile(fx,fy);
				Enemy *pNME=new Enemy(sprRoller,ENEMY_ROLLER,fx,fy);				
				AddEntity(pNME);
				nEnemies++;
			}
		}			
	}

	
}


void CGame::LoadTextures(void)
{
	GL_CreateRenderTargets();

	GL_BindRenderTarget(OSRT_LIGHTING);
	GL_LoadTexture(0,"pointlight.png");
	GL_LoadTexture(1,"spotlight.png");
	GL_LoadTexture(2,"triplelight.png");
		

	GL_LoadTexture(10,"backdrop.png");

	GL_LoadTexture(20,"tileset.png");
	
	for(int n=0;n<64;n++)
	{
		char szName[128];
		if (n<10) sprintf(szName,"player\\droid000%d.png",n);
			 else sprintf(szName,"player\\droid00%d.png",n);
		sprPlayer[n].Load(szName,32,32, 100+n);	
		sprPlayer[n].SetOrigin(16,16);
	}

	sprCeilLight.Load("ceillight.png",32,32,40);
	sprCeilLight.SetOrigin(16,8);

	
	sprSpinLight.Load("rotlight.png",32,32,41);
	sprSpinLight.SetOrigin(16,16);

	sprTurret.Load("player\\gun.png",32,32, 51);	
	sprTurret.SetOrigin(16,16);

	sprBullet.Load("bullet.png",16,16, 61);	
	sprBullet.SetOrigin(8,8);


	for(int n=0;n<64;n++)
	{
		char szName[128];
		if (n<10) sprintf(szName,"enemy\\floater000%d.png",n);
			 else sprintf(szName,"enemy\\floater00%d.png",n);
		sprFloater[n].Load(szName,32,32, 200+n);	
		sprFloater[n].SetOrigin(16,16);
				
		if (n<10) sprintf(szName,"enemy\\crawler000%d.png",n);
			 else sprintf(szName,"enemy\\crawler00%d.png",n);
		sprCrawler[n].Load(szName,32,32, 300+n);	
		sprCrawler[n].SetOrigin(16,16);

		if (n<10) sprintf(szName,"enemy\\roller000%d.png",n);
			 else sprintf(szName,"enemy\\roller00%d.png",n);
		sprRoller[n].Load(szName,32,32, 400+n);	
		sprRoller[n].SetOrigin(16,16);
	}


	sprExplode.Load("explode.png",32,32, 62);	
	sprExplode.SetOrigin(16,16);
	sprPtlLight.Load("ptllight.png",64,64, 63);	
	sprPtlLight.SetOrigin(32,32);


	GL_BindRenderTarget(OSRT_SCREEN);
}



void CGame::RenderLights()
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);


	// Render the lights
	static float fTimer=0;
	fTimer+=0.02f;

}



void CGame::RenderMap(bool boTop)
{

	float fTS=0.25f;

	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0.5f);



	
	GL_BindTexture(20);
  

	int x0=(fScrollX/32)-1;
	int y0=(fScrollY/32)-1;
	int x1=x0+22;
	int y1=y0+18;

	if (x0<0) x0=0;  
	if (y0<0) y0=0;
	if (x1>64) x1=64;  
	if (y1>64) y1=64;

	for(int y=y0;y<y1;y++)
		for(int x=x0;x<x1;x++)
		{
		   int t=gMap[x][y];

			
		   if (t>0)
			if (IsTileSolid(t)==boTop)
			{
				float fX=x*32;
				float fY=y*32;

				float fTX=0.25f* ((float)(t&3));
				float fTY=0.25f* ((float)(t>>2));

				if ((gMap[x][y])==0) fTX=0;

				glBegin(GL_QUADS);
				glColor4f(1,1,1,1);
				glTexCoord2f(fTX,fTY);
				glVertex2f(fX,fY);
				glTexCoord2f(fTX+fTS,fTY);
				glVertex2f(fX+32,fY);
				glTexCoord2f(fTX+fTS,fTY+fTS);
				glVertex2f(fX+32,fY+32);
				glTexCoord2f(fTX,fTY+fTS);
				glVertex2f(fX,fY+32);
				glEnd();
			}
		}

	glDisable(GL_ALPHA_TEST);
}


void CGame::Process(void)
{	
	GL_BindRenderTarget(OSRT_LIGHTING);
	
	if (nState==STATE_ACTIVE || nState==STATE_LEVELOUTRO)
	{
		UpdateEntities();
	}

	// Update scrolling
	float fTX= pPlr->vPos.x-320;
	float fTY= pPlr->vPos.y-240;
	fScrollX=(0.95*fScrollX)+(0.05f*fTX);
	fScrollY=(0.95*fScrollY)+(0.05f*fTY);

	float fScrollXi=(float) ((int)fScrollX);
	float fScrollYi=(float) ((int)fScrollY);

	// some ambient for now
	glClearColor(0.0,0.0f,0.0f,1);

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	GL_SetViewport(0,0,WIDTH,HEIGHT,true);
	glTranslatef(-fScrollXi,-fScrollYi,0);

	RenderLights();
	RenderEntityLights();

	GL_BindRenderTarget(OSRT_SCREEN);			
	glClearColor(0.5f,0.5f,0.5f,1);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Draw the tilemap
	glDisable(GL_BLEND);
	GL_SetViewport(0,0,WIDTH,HEIGHT,true);
	
	
	fTX=0,fTY=0;
	fTX +=(fScrollX/256.0f);
	fTY +=(fScrollY/256.0f);

	GL_BindTexture(10);
	glBegin(GL_QUADS);
	glColor4f(1,1,1,1);
	glTexCoord2f(fTX,fTY);
	glVertex2f(0,0);
	glTexCoord2f(fTX+(640.0f/128.0f),fTY);
	glVertex2f(640,0);
	glTexCoord2f(fTX+(640.0f/128.0f),fTY+(480.0f/128.0f));
	glVertex2f(640,480);
	glTexCoord2f(fTX,fTY+(480.0f/128.0f));
	glVertex2f(0,480);
	glEnd();

	glTranslatef(-fScrollXi,-fScrollYi,0);

	RenderMap(false);	// back layer
	RenderEntities();
	RenderMap(true);	// top layer

		

	GL_BindRenderTargetAsTexture(OSRT_LIGHTING);	
	GL_SetViewport(0,0,WIDTH,HEIGHT,true);

	glEnable(GL_BLEND);
	glBlendFunc(GL_DST_COLOR,GL_ZERO);	
	glColor4f(1,1,1,1);

	// Blit the light map over it
	glBegin(GL_QUADS);	
	glTexCoord2f(0,480.0f/512.0f);
	glVertex2f(0,0);	
	
	glTexCoord2f(640.0f/1024.0f,480.0f/512.0f);
	glVertex2f(640,0);
	
	glTexCoord2f(640.0f/1024.0f,0);
	glVertex2f(640,480);
	
	glTexCoord2f(0,0);
	glVertex2f(0,480);
	glEnd();


	glDisable(GL_BLEND);
	if (nState!=STATE_TITLE)
	{
		char szTemp[64];
				
		GL_SetViewport(0,0,WIDTH,HEIGHT,true);
		glColor4f(1,1,1,0.5f);
		sprintf(szTemp,"Sector #%d",nLevel);
		GL_PrintText(szTemp,10,20,1);
		glColor4f(0.8f,1,1,0.5f);
		sprintf(szTemp,"Score: %06d",nScore);
		GL_PrintText(szTemp,15,35,1 );
		glColor4f(0.6f,1,1,0.5f);	
		sprintf(szTemp,"Lives: %02d",nLives);
		GL_PrintText(szTemp,20,50,1 );
		glColor4f(0.4f,1,1,0.5f);
		sprintf(szTemp,"Kills: %03d/%03d",nKills,nEnemies);
		GL_PrintText(szTemp,25,65,1 );
	}
	else
	{
		GL_PrintText("F.U.B.A.R.",540,35,0 );		
		GL_PrintText("(FUBAR Unit: Bloody Aggressive Robot)",400,50,1 );		
		

		GL_PrintText("<Fire> = Space",15,385,1 );
		GL_PrintText("<Move> = Arrows",15,400,1 );
		GL_PrintText("<Jump> = Up Arrow",15,415,1 );
		GL_PrintText("<Aim>  = Z/X",15,430,1 );
		GL_PrintText("Press <Fire> To Start!",15,450,0 );

		if (KEY_Pressed(KEY_FIRE))
		{
			nLevel++;
			InitLevel();
			nState=STATE_LEVELINTRO;
			fStateTimer=3.0f;
		}
	}

	if (   (nState==STATE_LEVELINTRO)
		|| (nState==STATE_LEVELOUTRO))
	{
		fStateTimer-=0.015f;

		glColor4f(0.8f,1,1,0.5f);

		if (nState==STATE_LEVELINTRO)
		{
			char szTemp[128];
			sprintf(szTemp,"SECTOR #%d - %s",nLevel, szLevelNames[nLevel]);
			GL_PrintText(szTemp,15,432,0 );
			glColor4f(0.6f,1,1,0.5f);
			GL_PrintText("Eliminate all hostile units!",15,450,0 );
		}
		else
		{
			if (nKills<nEnemies)
			{
				if (nLives==0)
					GL_PrintText("Game Over!",15,450,0 );
				//else
				//	GL_PrintText("",15,450,0 );
			}
			else
				GL_PrintText("Sector Cleared!",15,450,0 );
		}
		
		if (fStateTimer<0)
		{
			if (nState==STATE_LEVELINTRO)
			{				
				nState=STATE_ACTIVE;
			}
			
			if (nState==STATE_LEVELOUTRO)
			{				
				if (nKills>=nEnemies)
				{
					nLevel++;
					InitLevel();
					nState=STATE_LEVELINTRO;
					fStateTimer=3.0f;
				}
				else
				{
					nLives--;
					if (nLives>=0)
					{
						InitLevel();
						nState=STATE_LEVELINTRO;
						fStateTimer=3.0f;
					}
					else
					{
						nLevel=0;
						InitLevel();
						nState=STATE_TITLE;
					}
				}
			}
		}
	}

	if (nState==STATE_ACTIVE)
	{
		// Handle player death
		if (!pPlr->bAlive)
		{
			nState=STATE_LEVELOUTRO;
			fStateTimer=3.0f;
		}

		// Handle level complete
		if (nKills>=nEnemies)
		{
			nState=STATE_LEVELOUTRO;
			fStateTimer=3.0f;
		}
	}

}


void CGame::Deinit(void)
{
}


